/**
 ** sparkCntrlTaskCodels.cc
 **
 ** Codels used by the control task sparkCntrlTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <portLib.h>

#include "server/sparkHeader.h"
#include "sparkm3d.h"



/*------------------------------------------------------------------------
 * sparkInterfaceAgenParamsCtrl  -  control codel of CONTROL request SetInterfaceAgentParams
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
sparkInterfaceAgenParamsCtrl(SPARK_INTERFACE_AGENT_PARAMS *interfaceAgentParams, int *report)
{
  int i;
  
  if(gAgents == NULL) 
    return ERROR;

  for(i=0; i<gAgents->all_agents_no; i++) {
    if(strcasestr(gAgents->all_agents[i]->robotPt->name, interfaceAgentParams->agentName.name)){
      if(interfaceAgentParams->showFov) 
	gAgents->all_agents[i]->perspective->enable_vision_draw = TRUE;
      else 
	gAgents->all_agents[i]->perspective->enable_vision_draw = FALSE;	
      
      if(interfaceAgentParams->showPoiting) 
	gAgents->all_agents[i]->perspective->enable_pointing_draw = TRUE;
      else 
	gAgents->all_agents[i]->perspective->enable_pointing_draw = FALSE;

      if(interfaceAgentParams->showVisibleObjects) 
	gAgents->all_agents[i]->perspective->enable_visible_objects_draw = TRUE;
      else 
	gAgents->all_agents[i]->perspective->enable_visible_objects_draw = FALSE;
    }
  }
  
  return OK;
}


/*------------------------------------------------------------------------
 * sparkSetInterfaceMightParamsCtrl  -  control codel of CONTROL request SetInterfaceMightParams
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
sparkSetInterfaceMightParamsCtrl(SPARK_INTERFACE_MIGHTABILITY_PARAMS *interfaceMightParams, int *report)
{
  // TODO:Fix mightabilities to compile with the latest version of BioMove3D (3.7.8)
  // SHOW_MM_BASED_OBJECT_VISIBLE = interfaceMightParams->showVisibleObjects;
  // SHOW_MM_BASED_OBJECT_REACHABLE = interfaceMightParams->showReachableObjects;
  
  return OK;
}


/*------------------------------------------------------------------------
 * sparkDisplayAgentPerspectiveCtrl  -  control codel of CONTROL request DisplayAgentPerspective
 *
 * Description:    
 * Report: OK
 *                 S_spark_INTERFACE_DISABLED
 *                 S_spark_NOT_INITIALIZED
 *                 S_spark_AGENT_NOT_FOUND
 *
 * Returns:    OK or ERROR
 */

STATUS
sparkDisplayAgentPerspectiveCtrl(GEN_STRING64 *agentName, int *report)
{
  int i;
  G3D_Window *window;
  
  if(gAgents == NULL) {
    *report = S_spark_NOT_INITIALIZED;
    return OK;
  }
  
  if(!SDI_F->p3d.enableGraphic) {
    *report = S_spark_INTERFACE_DISABLED;
    return OK;
  }
  
  for(i=0; i<gAgents->all_agents_no; i++) {
    if(strcasestr(gAgents->all_agents[i]->robotPt->name, agentName->name)) {
      window = g3d_get_win_by_name((char *)"Move3D");
      g3d_set_camera_parameters_from_frame(gAgents->all_agents[i]->perspective->camjoint->abs_pos,
					   window->vs);
      break;
    }
  }
  
  if(i==gAgents->all_agents_no) {
    *report = S_spark_AGENT_NOT_FOUND;
    return OK;
  }
  return OK;
}

/* sparkSetInterfaceParamsMain  -  codel EXEC of SetInterfaceParams
   Returns:  EXEC END ETHER FAIL ZOMBIE */
STATUS
sparkChangeInterfaceDisplayCtrl(SPARK_CHANGE_DISPLAY_PARAMS *interfaceChangeDisplayParams, int *report)
{
  G3D_Window *win = g3d_get_win_by_name((char *)"Move3D");
  
  if(!SDI_F->p3d.enableGraphic){
    printf("The Interface is disabled.\n");
    *report = S_spark_INTERFACE_DISABLED;
    return ETHER;
  }

  if ( interfaceChangeDisplayParams->interface_change_option == BB ) {
    if(interfaceChangeDisplayParams->is_option_true)
      win->vs.BB = TRUE;
    else
      win->vs.BB = FALSE;
	
  }
  else if ( interfaceChangeDisplayParams->interface_change_option == GHOST) {
    if(interfaceChangeDisplayParams->is_option_true)
      win->vs.GHOST = TRUE;
    else
      win->vs.GHOST = FALSE;      
  }
  return OK;
}
