/**
 ** sparkInterfaceTaskCodels.cc
 **
 ** Codels called by execution task sparkInterfaceTask
 **
 ** Author: Akin Sisbot
 ** Date: Aug 10
 **
 **/

#include <portLib.h>

#include "server/sparkHeader.h"
#include "sparkm3d.h"

#ifdef USE_GLUT
#include <GL/freeglut.h>
#endif

#ifdef QT_LIBRARY
#include "Qt-pkg.h"
#include <QtGui/QApplication>
#include <QtGui/QIcon>
#include <QtGui/QPixmap>
QApplication* gInterfaceApp = NULL;
GLWidget* gInterfaceOpenGlWidget = NULL;
#endif

void draw_spark_backbuffer()
{
#ifdef USE_GLUT
  g3d_glut_set_swap(false);
  g3d_glut_paintGL();
  g3d_glut_set_swap(true);
#else
#ifdef QT_LIBRARY
  gInterfaceOpenGlWidget->setAutoBufferSwapping(false);
  gInterfaceOpenGlWidget->updateGL(); 
  gInterfaceOpenGlWidget->setAutoBufferSwapping(true);
#endif
#endif
}

void initialize_interface() 
{
  G3D_Window *window;

#ifdef QT_LIBRARY
#ifdef BETA_INTERFACE
  
  char *argv[2];
  int argc;

  argv[0] = (char*) "Spark";
  argv[1] = (char*) "Toto";
  argc = 2; 

  gInterfaceApp = new QApplication(argc, argv);

#endif

  gInterfaceOpenGlWidget = new GLWidget(NULL);
  gInterfaceOpenGlWidget->setObjectName(QString::fromUtf8("Spark OpenGL"));
  gInterfaceOpenGlWidget->show();
  gInterfaceOpenGlWidget->raise();
  
  ext_g3d_draw_allwin_active_backbuffer =  (void (*)())(draw_spark_backbuffer);

#endif

  window = g3d_get_win_by_name((char *)"Move3D");
  g3d_set_win_floor_color(window->vs, 1, 0.87, 0.49);
  window->vs.displayFloor = TRUE;
  window->vs.displayWalls = TRUE;

  ext_g3d_draw_allwin_active_backbuffer =  draw_spark_backbuffer;
  printf("ext_g3d_draw_allwin_active_backbuffer = %d" , ext_g3d_draw_allwin_active_backbuffer);
}

void interface_MainLoopEvent()
{
#ifdef USE_GLUT
   glutMainLoopEvent ();
#endif
}

/* sparkUpdateInterfaceMain  -  codel EXEC of UpdateInterface
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
sparkUpdateInterfaceMain(int *report)
{  
  if(SDI_F->p3d.enableGraphic){
    //  GENOM_UNLOCK_SDI;  

#ifdef USE_GLUT
    glutPostRedisplay();
#else
#ifdef QT_LIBRARY
    gInterfaceOpenGlWidget->updateGL();
#endif
#ifdef BETA_INTERFACE
    
    // gInterfaceApp->processEvents();
#endif    
#endif

    //if(SDI_F->interfaceParams.saveInterface){
      //g3d_screenshot((char *)"Move3D");
      // TODO: put qt save window function    
    //}
    
    //  GENOM_LOCK_SDI;
    *report = OK;
    return EXEC;
  }
  else{
    printf("Graphics not enabled\n");
    *report = S_spark_INTERFACE_DISABLED;
    return ETHER;
  }
}


/* sparkChangeCameraPosMain  -  codel EXEC of ChangeCameraPos
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
sparkChangeCameraPosMain(SPARK_CAM_POS *camPos, int *report)
{
  G3D_Window *window = g3d_get_win_by_name((char *)"Move3D");
  
  if(!SDI_F->p3d.enableGraphic){
    printf("The Interface is disabled. Cannot change the camera position.\n");
    *report = S_spark_INTERFACE_DISABLED;
    return ETHER;
  }
  
  if(window){
    printf("Changing camera position of the window: %s\n",window->name);
    g3d_set_win_camera(window->vs, camPos->xdest, camPos->ydest, camPos->zdest,
		       camPos->dist, camPos->hrot, camPos->vrot, 0.0, 0.0, 1.0);
  }
  
  *report = OK;
  return ETHER;
}



/* sparkSetInterfaceParamsMain  -  codel EXEC of SetInterfaceParams
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
sparkSetInterfaceParamsMain(SPARK_INTERFACE_PARAMS *interfaceParams, int *report)
{
  G3D_Window *win = g3d_get_win_by_name((char *)"Move3D");
  
  if(!SDI_F->p3d.enableGraphic){
    printf("The Interface is disabled.\n");
    *report = S_spark_INTERFACE_DISABLED;
    return ETHER;
  }
  

  if( (interfaceParams->width > 0) &&  (interfaceParams->height > 0) )
    {
#ifdef QT_LIBRARY
    gInterfaceOpenGlWidget->setWinSize(interfaceParams->width, interfaceParams->height);
#endif
#ifdef USE_GLUT
    glutReshapeWindow(interfaceParams->width,interfaceParams->height);
#endif
    }


  if(!interfaceParams->floor) 
    win->vs.displayFloor = FALSE;
  else 
    win->vs.displayFloor = TRUE;
  
  if(!interfaceParams->walls) 
    win->vs.displayWalls = FALSE;
  else 
    win->vs.displayWalls = TRUE;
  
  if(!interfaceParams->shadows) 
    win->vs.displayShadows = FALSE;
  else 
    win->vs.displayShadows = TRUE;

 if(!interfaceParams->tiles) 
    win->vs.displayTiles = FALSE;
  else 
    win->vs.displayTiles = TRUE;

 if(!interfaceParams->antialiasing) 
    win->vs.enableAntialiasing = FALSE;
  else 
    win->vs.enableAntialiasing = TRUE;
  
  *report = OK;
  return ETHER;
}

