/*****************************************************************************
*
* Emrah Akin Sisbot
* Aug 10 
*
* File : sparkOro.h
* Brief: This file contains functions for SPARK - Ontology link
*
******************************************************************************/


#ifdef ORO

#include "server/sparkHeader.h"
#include "sparkm3d.h"

#include <string>
#include <map>
#include <iostream>
#include <stdlib.h>
#include <portLib.h>
#include <set>

// These are the common includes for the ontology:
// oro.h defines the ontology API
#include "liboro/oro.h"
// oro_library.h is automatically generated from the oro ontology
// and contains all the concepts defined in it.
#include "liboro/oro_library.h"
// oro_exceptions.h defines the exceptions possibily generated by liboro.
#include "liboro/oro_exceptions.h"
// yarp_connector.h is an implementation of a conenctor to the server
// based on the YARP middleware.
#include "liboro/socket_connector.h"

using namespace std;
using namespace oro;

static IConnector *connector;
static Ontology *onto; //the accessor to the ontology singleton.

typedef struct agent_stmt {
  string agentID;
  set<Statement> stmtsToUpdate;
  set<string> stmtsToClear;
  set<Statement> stmtsToRemove;
  set<Statement> stmtsToAdd;
} agent_stmt;

vector<agent_stmt> agents_stmts;

int connectOro(char *host, char *port)
{
  
  //initialize the connection to the ontology server.
  string oro_server_host(host);
  string oro_server_port(port); 
  
  try {
    connector = new SocketConnector(oro_server_host, oro_server_port);
    onto = Ontology::createWithConnector(*connector);
  } catch (ConnectorException ce) {
    // Connection problem
    return -1;
  } catch (OntologyServerException ose) {
    // Server problem
    return -2;
  }
  printf("Connected to the Ontology Server!\n");
  return 1;
} 

// int updateAgentPosture(HRI_AGENT * agent, int agent_i, SPARK_HUMAN_STATE_ENUM state)
// {
//   char tempBuffer[100];

//   if ( !agents_stmts.empty() ) {
    
//     if(state == SITTING)
//       sprintf(tempBuffer,"%s isSitting true", agent->robotPt->name);
//     else
//       sprintf(tempBuffer,"%s isSitting false", agent->robotPt->name);
    
//     agents_stmts[agent_i].stmtsToUpdate.insert(Statement(tempBuffer));
//   }
// }

// int updateAgentSp(int agent_i, HRI_AGENT * tagent, SPARK_SPATIAL_RELATION spatialRelation)
// { 
//   char tempBuffer[100];
  
//   if ( !agents_stmts.empty() ) {

//     switch(spatialRelation){
//     case NEAR_FRONT:       {sprintf(tempBuffer,"%s isLocated NEAR_FRONT", tagent->robotPt->name ); break;}
//     case NEAR_FRONT_LEFT:  {sprintf(tempBuffer,"%s isLocated NEAR_LEFT", tagent->robotPt->name); break;}
//     case NEAR_LEFT:        {sprintf(tempBuffer,"%s isLocated NEAR_LEFT", tagent->robotPt->name); break;}
//     case NEAR_BACK_LEFT:   {sprintf(tempBuffer,"%s isLocated NEAR_LEFT", tagent->robotPt->name); break;}
//     case NEAR_BACK:        {sprintf(tempBuffer,"%s isLocated NEAR_BACK", tagent->robotPt->name); break;}
//     case NEAR_BACK_RIGHT:  {sprintf(tempBuffer,"%s isLocated NEAR_RIGHT",tagent->robotPt->name ); break;}
//     case NEAR_RIGHT:       {sprintf(tempBuffer,"%s isLocated NEAR_RIGHT",tagent->robotPt->name ); break;}
//     case NEAR_FRONT_RIGHT: {sprintf(tempBuffer,"%s isLocated NEAR_RIGHT",tagent->robotPt->name); break;}
//     case FAR_FRONT:        {sprintf(tempBuffer,"%s isLocated FAR_FRONT", tagent->robotPt->name); break;}
//     case FAR_FRONT_LEFT:   {sprintf(tempBuffer,"%s isLocated FAR_LEFT",tagent->robotPt->name) ; break;}
//     case FAR_LEFT:         {sprintf(tempBuffer,"%s isLocated FAR_LEFT",tagent->robotPt->name ); break;}
//     case FAR_BACK_LEFT:    {sprintf(tempBuffer,"%s isLocated FAR_LEFT",tagent->robotPt->name ); break;}
//     case FAR_BACK :        {sprintf(tempBuffer,"%s isLocated FAR_BACK",tagent->robotPt->name ); break;}
//     case FAR_BACK_RIGHT:   {sprintf(tempBuffer,"%s isLocated FAR_RIGHT",tagent->robotPt->name ); break;}
//     case FAR_RIGHT:        {sprintf(tempBuffer,"%s isLocated FAR_RIGHT",tagent->robotPt->name ); break;}
//     case FAR_FRONT_RIGHT:  {sprintf(tempBuffer,"%s isLocated FAR_RIGHT",tagent->robotPt->name ); break;}
//     }
    
//     agents_stmts[agent_i].stmtsToUpdate.insert(Statement(tempBuffer));
//   }
// }

// int updateAgentAgentVisibility(HRI_AGENT * agent, int agent_i, HRI_AGENT * tagent, SPARK_VISIBILITY_PLACEMENT vispl, SPARK_VISIBILITY vis)
// {
//   char tempBuffer[100];

//   if ( !agents_stmts.empty() ) {
    
//     if(vis == VISIBLE)
//       sprintf(tempBuffer,"%s isVisible true", tagent->robotPt->name);
//     else
//       sprintf(tempBuffer,"%s isVisible false", tagent->robotPt->name);
    
//     agents_stmts[agent_i].stmtsToUpdate.insert(Statement(tempBuffer));
    
    
//     printf("agent %s, target %s, visibility %d, fov %d\n", agent->robotPt->name, tagent->robotPt->name,vis,vispl );

//     if( vis==VISIBLE && vispl == FOA) {
//       sprintf(tempBuffer,"%s looksAt ?val",
// 	      agent->robotPt->name);
//       agents_stmts[agent_i].stmtsToClear.insert(tempBuffer);	      
      
//       sprintf(tempBuffer,"%s looksAt %s",
// 	      agent->robotPt->name,
// 	      tagent->robotPt->name);
//       agents_stmts[agent_i].stmtsToAdd.insert(Statement(tempBuffer));
//     } 
//   }

// } 



// int updateObject(SPARK_OBJECT_STATE * objectState, SPARK_OBJECT_STATE * previousState)
// {
//   char tempBuffer[100];
//   int i, j;
//   char * robotName;
//   agent_stmt AS;

//   robotName = objectState->agentVisibilityState[0].agentName.name;
  
//   // initialization of sets
//   if ( agents_stmts.empty() ) {
//     for(i=0; i<objectState->agentVisibilityStateNb; i++){
//       AS.agentID = string(objectState->agentVisibilityState[i].agentName.name);
//       agents_stmts.push_back(AS);
//     }
//   }
  
//   // Object has disappeared
//   if(objectState->disappeared) {
//     if(!previousState->disappeared) {    
//       sprintf(tempBuffer,"%s isMoving ?val", objectState->name.name);
//       agents_stmts[0].stmtsToClear.insert(tempBuffer);
    
//       for(i=0; i<objectState->agentVisibilityStateNb; i++){
      
// 	sprintf(tempBuffer,"%s isVisible ?val", objectState->name.name);
// 	agents_stmts[i].stmtsToClear.insert(tempBuffer);
	
// 	sprintf(tempBuffer,"%s isLocated ?val", objectState->name.name);
// 	agents_stmts[i].stmtsToClear.insert(tempBuffer);
	
// 	sprintf(tempBuffer,"%s looksAt ?val", objectState->agentVisibilityState[i].agentName.name);
// 	agents_stmts[0].stmtsToClear.insert(tempBuffer);
	
// 	sprintf(tempBuffer,"%s pointsAt ?val", objectState->agentVisibilityState[i].agentName.name);
// 	agents_stmts[0].stmtsToClear.insert(tempBuffer);
	
// 	sprintf(tempBuffer,"%s isReachable ?val", objectState->name.name );
// 	agents_stmts[i].stmtsToClear.insert(tempBuffer);	
//       }
      
     

//       sprintf(tempBuffer,"%s isIn ?val", objectState->name.name);
//       agents_stmts[0].stmtsToClear.insert(tempBuffer);
    
//       sprintf(tempBuffer,"%s isOn ?val", objectState->name.name);
//       agents_stmts[0].stmtsToClear.insert(tempBuffer);
      
//       sprintf(tempBuffer,"%s isNextTo ?val", objectState->name.name);
//       agents_stmts[0].stmtsToClear.insert(tempBuffer);
//     }

//     return TRUE;
//   }
  
//   // Begin to put things into ontology buffer
//   // onto->bufferize();
  
//   // Static - Moving
//   // if( previousState->disappeared ) { // if the motion has changed
    
//     if(objectState->motion == OBJ_MOVING)
//       sprintf(tempBuffer,"%s isMoving true", objectState->name.name);
//     else
//       sprintf(tempBuffer,"%s isMoving false", objectState->name.name);
//     agents_stmts[0].stmtsToUpdate.insert(Statement(tempBuffer));
//     //}
  
//   // Agent Visibility, Focus of attention and Spatial relation
  
//   for(i=0; i<objectState->agentVisibilityStateNb; i++) {
//     if( (previousState->agentVisibilityState[i].visibility != objectState->agentVisibilityState[i].visibility) ||
// 	previousState->disappeared ) { // if visibility state has changed or the obj was dissapeared
      
//       if(objectState->agentVisibilityState[i].visibility == VISIBLE)
// 	sprintf(tempBuffer,"%s isVisible true", objectState->name.name);
//       else
// 	sprintf(tempBuffer,"%s isVisible false", objectState->name.name);
//       agents_stmts[i].stmtsToUpdate.insert(Statement(tempBuffer));
      
//       if( (previousState->agentVisibilityState[i].visibilityPlacement != objectState->agentVisibilityState[i].visibilityPlacement) ||
// 	  previousState->disappeared ) { // if visibility placement has changed or the obj was dissapeared
	
// 	if(objectState->agentVisibilityState[i].visibility == VISIBLE && 
// 	   objectState->agentVisibilityState[i].visibilityPlacement == FOA) {
// 	  // Non fuctional statement - First Clear then add
// 	  sprintf(tempBuffer,"%s looksAt ?val",
// 		  objectState->agentVisibilityState[i].agentName.name);
// 	  agents_stmts[i].stmtsToClear.insert(tempBuffer);	      
	  
// 	  sprintf(tempBuffer,"%s looksAt %s",
// 		  objectState->agentVisibilityState[i].agentName.name,
// 		  objectState->name.name);
// 	  agents_stmts[i].stmtsToAdd.insert(Statement(tempBuffer));
// 	}
//       }
//     }
//     if( (previousState->agentVisibilityState[i].pointedAt != objectState->agentVisibilityState[i].pointedAt) ||
// 	previousState->disappeared ) { // if pointat has changed
//       if(objectState->agentVisibilityState[i].pointedAt) {
// 	// Non fuctional statement - First Clear then add
// 	sprintf(tempBuffer,"%s pointsAt ?val",
// 		objectState->agentVisibilityState[i].agentName.name);
// 	agents_stmts[i].stmtsToClear.insert(tempBuffer);	      
	
// 	sprintf(tempBuffer,"%s pointsAt %s",
// 		objectState->agentVisibilityState[i].agentName.name,
// 		objectState->name.name);
// 	agents_stmts[i].stmtsToAdd.insert(Statement(tempBuffer));
//       } 
//     }
    
//     // Spatial Relations
//     if( (previousState->agentVisibilityState[i].spatialRelation != objectState->agentVisibilityState[i].spatialRelation) ||
// 	previousState->disappeared ) {
//       switch(objectState->agentVisibilityState[i].spatialRelation){
//       case NEAR_FRONT:       {sprintf(tempBuffer,"%s isLocated FRONT", objectState->name.name); break;}
//       case NEAR_FRONT_LEFT:  {sprintf(tempBuffer,"%s isLocated LEFT", objectState->name.name); break;}
//       case NEAR_LEFT:        {sprintf(tempBuffer,"%s isLocated LEFT", objectState->name.name); break;}
//       case NEAR_BACK_LEFT:   {sprintf(tempBuffer,"%s isLocated LEFT", objectState->name.name); break;}
//       case NEAR_BACK:        {sprintf(tempBuffer,"%s isLocated BACK", objectState->name.name); break;}
//       case NEAR_BACK_RIGHT:  {sprintf(tempBuffer,"%s isLocated RIGHT", objectState->name.name); break;}
//       case NEAR_RIGHT:       {sprintf(tempBuffer,"%s isLocated RIGHT", objectState->name.name); break;}
//       case NEAR_FRONT_RIGHT: {sprintf(tempBuffer,"%s isLocated RIGHT", objectState->name.name); break;}
//       case FAR_FRONT:        {sprintf(tempBuffer,"%s isLocated FRONT", objectState->name.name); break;}
//       case FAR_FRONT_LEFT:   {sprintf(tempBuffer,"%s isLocated LEFT", objectState->name.name); break;}
//       case FAR_LEFT:         {sprintf(tempBuffer,"%s isLocated LEFT", objectState->name.name); break;}
//       case FAR_BACK_LEFT:    {sprintf(tempBuffer,"%s isLocated LEFT", objectState->name.name); break;}
//       case FAR_BACK :        {sprintf(tempBuffer,"%s isLocated BACK", objectState->name.name); break;}
//       case FAR_BACK_RIGHT:   {sprintf(tempBuffer,"%s isLocated RIGHT", objectState->name.name); break;}
//       case FAR_RIGHT:        {sprintf(tempBuffer,"%s isLocated RIGHT", objectState->name.name); break;}
//       case FAR_FRONT_RIGHT:  {sprintf(tempBuffer,"%s isLocated RIGHT", objectState->name.name); break;}
//       }
      
//       agents_stmts[i].stmtsToUpdate.insert(Statement(tempBuffer)); 
//     }
//   }
  
//  // Placement Relations
//   for(i=0; i<objectState->placementLength; i++) {
//     for(j=0; j<objectState->agentVisibilityStateNb; j++) {
//       if( (previousState->agentVisibilityState[j].visibility != objectState->agentVisibilityState[j].visibility) ||
// 	  (previousState->agentVisibilityState[j].visibilityPlacement != objectState->agentVisibilityState[j].visibilityPlacement) ||
// 	  previousState->disappeared ) {
	
// 	if(objectState->agentVisibilityState[j].visibility == VISIBLE && 
// 	   objectState->agentVisibilityState[j].visibilityPlacement == FOA) {
// 	  //TODO: Ideally we have to test if both objects are visible for an agent.
	  
// 	  if(objectState->placements[i].relation == ISIN)
// 	    sprintf(tempBuffer,"%s isIn %s", objectState->name.name, objectState->placements[i].object.name);
// 	  if(objectState->placements[i].relation == ISON)
// 	    sprintf(tempBuffer,"%s isOn %s", objectState->name.name, objectState->placements[i].object.name);
// 	  if(objectState->placements[i].relation == ISNEXTTO)
// 	    sprintf(tempBuffer,"%s isNextTo %s", objectState->name.name, objectState->placements[i].object.name);
	  
// 	  agents_stmts[j].stmtsToUpdate.insert(Statement(tempBuffer));
// 	} 
//       }
//     }
//   }
  
//   // With this final flush, things are sent to oro-server
//   // onto->flush();
  
//   return TRUE;
// }

int sendToORO(GEN_STRING64 address, GEN_STRING64 port)
{
  int i;
  set<string>::iterator it;
  set<Statement>::iterator itu;

  try {
    if(!SDI_F->state.oroConnectionAv) {
      if ( connectOro(address.name, port.name) == 1 ) {
	SDI_F->state.oroConnectionAv = TRUE;
      }
      else {
	return FALSE;
      }
    }
    
    //cout << "TIME:" << time(0) << endl;
    for ( i = 0; i < agents_stmts.size() ; i++) {
      //cout << "FOR AGENT " << agents_stmts[i].agentID << ":" << endl;
      if ( !agents_stmts[i].stmtsToClear.empty()) {	
	// cout << "\tCLEAR contains:" << endl;
	// for ( it=agents_stmts[i].stmtsToClear.begin() ; it != agents_stmts[i].stmtsToClear.end(); it++ )
	//   cout << "\t " << *it << endl;
	// cout << endl;	
	//-------------
	onto->clearForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToClear);
	agents_stmts[i].stmtsToClear.clear();
	//-------------
      }
 
      if ( !agents_stmts[i].stmtsToRemove.empty()) {
	
	// cout << "\tRemove contains:" << endl;
	// for ( itu=agents_stmts[i].stmtsToRemove.begin() ; itu != agents_stmts[i].stmtsToRemove.end(); itu++ )
	//   cout << "\t " << *itu << endl;
	// cout << endl;
	
	//-------------
       onto->removeForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToRemove);
       agents_stmts[i].stmtsToRemove.clear();
	//-------------
      }
     
      if ( !agents_stmts[i].stmtsToAdd.empty()) {	
	// cout << "\tADD contains:" <<endl;
	// for ( itu=agents_stmts[i].stmtsToAdd.begin() ; itu != agents_stmts[i].stmtsToAdd.end(); itu++ )
	//   cout << "\t " << *itu << endl;
	// cout << endl;	
	//------------
	onto->addForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToAdd);
	agents_stmts[i].stmtsToAdd.clear();
	//------------
      }
      
      if ( !agents_stmts[i].stmtsToUpdate.empty()) {
	// cout << "\tUPDATE contains:" <<endl;
	// for ( itu=agents_stmts[i].stmtsToUpdate.begin() ; itu != agents_stmts[i].stmtsToUpdate.end(); itu++ )
	//   cout << "\t " << *itu << endl;
	// cout << endl;
	//-----------
	onto->updateForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToUpdate);
	agents_stmts[i].stmtsToUpdate.clear();
	//-----------
      }
    }
  } catch (const ConnectorException& ce) {
    // Connection problem
    printf("%s\n",ce.what());
    SDI_F->state.oroConnectionAv = FALSE;
    return FALSE;
  } catch (const OntologyServerException& ose) {
    // Server problem
    printf("%s\n",ose.what());
    SDI_F->state.oroConnectionAv = FALSE;
    return FALSE;
  }
  return TRUE;
}


// int sendReachabilitiesToORO(SPARK_ALL_REACHABILITIES *reachabilities, GEN_STRING64 address, GEN_STRING64 port)
// {
//   int i;
//   char tempBuffer[100];
  
//   try {
//     if(!SDI_F->state.oroConnectionAv) {
//       if ( connectOro(address.name, port.name) == 1 ) {
// 	SDI_F->state.oroConnectionAv = TRUE;
//       }
//       else {
// 	SDI_F->state.oroConnectionAv = FALSE;
// 	return FALSE;
//       }
//     }
    
//     for(i=0; i<reachabilities->agentReachNb; i++) {
//       switch(reachabilities->agentReach[i].reachability) {
//       case REACHABILITY_UNKNOWN: 
// 	sprintf(tempBuffer,"%s isReachable ?val", reachabilities->agentReach[i].objectName.name);
// 	//onto->clearForAgent(reachabilities->agentReach[i].agentName.name, stmts);
// 	//onto->removeForAgent("myself", Statement(tempBuffer));
// 	break;
//       case UNREACHABLE:
// 	sprintf(tempBuffer,"%s isReachable false", reachabilities->agentReach[i].objectName.name);
// 	onto->updateForAgent(reachabilities->agentReach[i].agentName.name, Statement(tempBuffer));
// 	//onto->updateForAgent("myself", Statement(tempBuffer));
// 	break;
//       case REACHABLE:
// 	sprintf(tempBuffer,"%s isReachable true", reachabilities->agentReach[i].objectName.name); 
//       	onto->updateForAgent(reachabilities->agentReach[i].agentName.name, Statement(tempBuffer));
// 	//onto->updateForAgent("myself", Statement(tempBuffer));
// 	break;
//       case REACHABLE_WITH_COLLISION:
// 	sprintf(tempBuffer,"%s isReachable true", reachabilities->agentReach[i].objectName.name); 
//       	onto->updateForAgent(reachabilities->agentReach[i].agentName.name, Statement(tempBuffer));
// 	//onto->updateForAgent("myself", Statement(tempBuffer));
// 	break;      
//       }
//     }
//   } catch (const ConnectorException& ce) {
//     // Connection problem
//     printf("%s\n",ce.what());
//     SDI_F->state.oroConnectionAv = FALSE;
//     return FALSE;
//   } catch (const OntologyServerException& ose) {
//     // Server problem
//     printf("%s\n",ose.what());
//     SDI_F->state.oroConnectionAv = FALSE;
//     return FALSE;
//   }
  
//   return TRUE;
// }

// Modification added by Matthieu Warnier 17/01/2011
// To avoid sending information forever, I have included the is_changed and 
// is_exported bool. Information is sent only if we have both is_changed TRUE 
// ( value changed since the last computation ) and is_exported FALSE
// ( It was not yet exported. ) is_exported is put to TRUE after export
int prepareFactsForORO(HRI_AGENTS * agents, HRI_ENTITIES * ents)
{  
  char tempBuffer[100];
  int a_i, e_i, e_j;
  agent_stmt AS;
  HRI_KNOWLEDGE_ON_ENTITY * kn_on_ent;
  HRI_AGENT * agent;

  // For testing purpose we populate this temporaraly deprecated poster with visibility value for tests scripts. 
  int needToExportObjStatePoster; 
  int visibilitySparkToBioConv;
 

  needToExportObjStatePoster = FALSE;

  // initialization of sets
  if ( agents_stmts.empty() ) {
    for(a_i=0; a_i<agents->all_agents_no; a_i++){
      AS.agentID = string(agents->all_agents[a_i]->robotPt->name);
      agents_stmts.push_back(AS);
    }
  }

  
  for(a_i=0; a_i<agents->all_agents_no; a_i++) {
    
    agent = agents->all_agents[a_i];
    
    // If the agent is in the scene
    if(agent->is_present) {
      
      for(e_i=0; e_i<agent->knowledge->entities_nb; e_i++) {
	
	kn_on_ent = &agent->knowledge->entities[e_i];

	// If the entity is a part of the current agent, we skip it since it doesn't make sense to compute it from his own point of view
	// TODO: Or does it?
	if( (kn_on_ent->entPt->type == HRI_AGENT_PART) || (kn_on_ent->entPt->type == HRI_ISAGENT) ) {
	  if( agent == agents->all_agents[kn_on_ent->entPt->agent_idx] )
	    continue;
	}	

	// If the entity is present in the scene
	if(kn_on_ent->entPt->is_present) {
	  
	  // Check if the entity has disappeared
	  
	  if(kn_on_ent->entPt->disappeared) {
	    if( !kn_on_ent->disappeared_isexported) {

	      // update isAt unknownLocation
	      sprintf(tempBuffer,"%s isAt unknownLocation", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));   
	      // 
	      kn_on_ent->disappeared_isexported = TRUE;
	    }
	  }

	  // delete isAt unknown location
	  if( !kn_on_ent->disappeared_isexported) {
	    sprintf(tempBuffer,"%s isAt ?val",kn_on_ent->entPt->name);
	    agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	    kn_on_ent->disappeared_isexported = TRUE;
	  }

	  // Static - Moving
	  if ( kn_on_ent->motion_ischanged && !kn_on_ent->motion_isexported) {
	    switch(kn_on_ent->motion) {
	    case HRI_MOVING:
	      sprintf(tempBuffer,"%s isMoving true", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;    
	    case HRI_STATIC: 
	      sprintf(tempBuffer,"%s isMoving false", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_UK_MOTION: 
	      sprintf(tempBuffer,"%s isMoving ?val", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	      break;
	    }
	    kn_on_ent->motion_isexported = TRUE;
	  }

	  // Visibile - Invisible
	  if ( kn_on_ent->visibility_ischanged && !kn_on_ent->visibility_isexported) {
	    switch(kn_on_ent->visibility) {
	    case HRI_VISIBLE:
	      sprintf(tempBuffer,"%s isVisible true", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_INVISIBLE:
	      sprintf(tempBuffer,"%s isVisible false", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_UK_VIS:
	      sprintf(tempBuffer,"%s isVisible ?val", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	      break;
	    }
	    kn_on_ent->visibility_isexported = TRUE;
	    
	    // Populate Poster for testing Purpose
	    if(e_i <SPARK_OBJ_STATE_MAX){
	      strcpy(SDI_F->objStates.object[e_i].name.name, kn_on_ent->entPt->name); 
	      strcpy(SDI_F->objStates.object[e_i].agentVisibilityState[a_i].agentName.name, agent->robotPt->name);
	      visibilitySparkToBioConv = (int) kn_on_ent->visibility;
	      visibilitySparkToBioConv--;
	      visibilitySparkToBioConv = visibilitySparkToBioConv * (-1);
	      SDI_F->objStates.object[e_i].agentVisibilityState[a_i].visibility = (SPARK_VISIBILITY) visibilitySparkToBioConv;
	      SDI_F->objStates.object[e_i].agentVisibilityState[a_i].visibilityPlacement = (SPARK_VISIBILITY_PLACEMENT) kn_on_ent->is_placed_from_visibility;
	      needToExportObjStatePoster = TRUE;
	    }
	  }
	  // Reachability - Commented out for performance reasons
	  // TODO: Make a difference in ORO for HARDLY_REACHABLE 
	  if ( kn_on_ent->reachability_ischanged && !kn_on_ent->reachability_isexported) {	    
	    switch(kn_on_ent->reachability) {
	    case HRI_REACHABLE:
	      sprintf(tempBuffer,"%s isReachable true", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_HARDLY_REACHABLE:
	      sprintf(tempBuffer,"%s isReachable true", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;  
	    case HRI_UNREACHABLE:
	      sprintf(tempBuffer,"%s isReachable false", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;	      
	    case HRI_UK_REACHABILITY:
	      sprintf(tempBuffer,"%s isReachable ?val", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);	     
	      break;	      
	    }
	    kn_on_ent->reachability_isexported = TRUE;
	  }
	  
	  // FOA - FOV - OOF
	  // TODO: Make ORO accept FOV-FOZ-OOF information
	    

	    
	  // LOOKSAT
	  if ( kn_on_ent->is_looked_at_ischanged && !kn_on_ent->is_looked_at_isexported) {
	    if(kn_on_ent->is_looked_at == HRI_TRUE_V) {	      	      
	      sprintf(tempBuffer,"%s looksAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	    }
	    else {
	      sprintf(tempBuffer,"%s looksAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));		
	    }
	    kn_on_ent->is_looked_at_isexported = TRUE;
	  }

	  // POINTSAT
	  if ( kn_on_ent->is_pointed_at_ischanged && !kn_on_ent->is_pointed_at_isexported) {
	    if(kn_on_ent->is_pointed_at == HRI_TRUE_V) {	      	      
	      sprintf(tempBuffer,"%s pointsAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	    }
	    else {
	      sprintf(tempBuffer,"%s pointssAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));		
	    }
	    kn_on_ent->is_pointed_at_isexported = TRUE;
	  }
	    
	  // FRONT - BACK - ETC
	  if ( kn_on_ent->spatial_relation_ischanged && !kn_on_ent->spatial_relation_isexported) {
	    switch(kn_on_ent->is_located_from_agent) { 
	    case HRI_NEAR_FRONT:       
	      sprintf(tempBuffer,"%s isLocated NEAR_FRONT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer)); 
	      break;
	    case HRI_NEAR_FRONT_LEFT:  
	      sprintf(tempBuffer,"%s isLocated NEAR_LEFT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_NEAR_LEFT:        
	      sprintf(tempBuffer,"%s isLocated NEAR_LEFT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_NEAR_BACK_LEFT:   
	      sprintf(tempBuffer,"%s isLocated NEAR_LEFT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_NEAR_BACK:        
	      sprintf(tempBuffer,"%s isLocated NEAR_BACK", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_NEAR_BACK_RIGHT:  
	      sprintf(tempBuffer,"%s isLocated NEAR_RIGHT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_NEAR_RIGHT:       
	      sprintf(tempBuffer,"%s isLocated NEAR_RIGHT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_NEAR_FRONT_RIGHT: 
	      sprintf(tempBuffer,"%s isLocated NEAR_RIGHT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_FRONT:        
	      sprintf(tempBuffer,"%s isLocated FAR_FRONT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_FRONT_LEFT:   
	      sprintf(tempBuffer,"%s isLocated FAR_LEFT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_LEFT:         
	      sprintf(tempBuffer,"%s isLocated FAR_LEFT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_BACK_LEFT:    
	      sprintf(tempBuffer,"%s isLocated FAR_LEFT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_BACK :        
	      sprintf(tempBuffer,"%s isLocated FAR_BACK", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_BACK_RIGHT:   
	      sprintf(tempBuffer,"%s isLocated FAR_RIGHT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_RIGHT:        
	      sprintf(tempBuffer,"%s isLocated FAR_RIGHT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_FAR_FRONT_RIGHT:  
	      sprintf(tempBuffer,"%s isLocated FAR_RIGHT", kn_on_ent->entPt->name); 
	      agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	      break;
	    case HRI_UK_RELATION :
	      sprintf(tempBuffer,"%s isLocated ?val", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	      break;
	    }
	    kn_on_ent->spatial_relation_isexported = TRUE;
	  }

	  // IN - ON - NEXTTO
	    
	  for(e_j=0; e_j<kn_on_ent->is_placed_nb; e_j++) {
	    // placement relation between the same object has no meaning
	    if ( ( e_j != e_i) && kn_on_ent->placement_relation_ischanged[e_j] && !kn_on_ent->placement_relation_isexported[e_j]) {
	      // delete previous value. ( right now only on placement relation is possible )
	      switch(kn_on_ent->is_placed_old[e_j]){
	      case HRI_UK_PLR:
		//Nothing to do
		break;
	      case HRI_ISON:
		sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
		agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
		break;
	      case HRI_ISNEXTTO:
		sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
		agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
		break;
	      case HRI_ISIN:
		sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
		agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
		break;
	      }
	    
	      switch(kn_on_ent->is_placed[e_j]) {
	      case HRI_ISIN: 
		sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
		agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
		break;
	      case HRI_ISON: 
		sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
		agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
		break;
	      case HRI_ISNEXTTO: 
		sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
		agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
		break;
	      case HRI_UK_PLR:
		//Nothing to do
		break;
	      }
	      kn_on_ent->placement_relation_isexported[e_j] = TRUE;	    
	    }
	  }   
	}
      }
    }
  }
  if(needToExportObjStatePoster)
    sparkObjectStatesobjStatesPosterWrite(SPARK_OBJECTSTATES_POSTER_ID, &SDI_F->objStates);
}



#endif
